/*
 --  ---------------------------------------------------------------------------
 --
 --      ISERVER  -  INMOS standard file server
 --
 --      hostc.c
 --
 --      Primary environment operations
 --
 --      Copyright (c) INMOS Ltd., 1988.
 --      All Rights Reserved.
 --
 --	derived from 1.41 - removed UDP support, added MIPS etc.
 --	29-sep-92 lr	some cleanup
 --	21-Jul-92 lr
 --	   warning: polling the keyboard stops the server for 0.1 sec
 --	   if no keys are present.
 --	   Terminal I/O still flaky
 --	05-Jun-92 lr
 --	   now tty support works better ?
 --     18-July-2000	Kev
 --        Port to Linux
 --  ---------------------------------------------------------------------------
*/



#include <stdio.h>
#include <string.h>
#include <time.h>
#include <stdlib.h>

#include <sys/termios.h>

#include "inmos.h"
#include "iserver.h"
#include "pack.h"


/**** Point out those things that live outside this module ****/

PRIVATE BYTE DataBuffer[MAX_SLICE_LENGTH+1];
PRIVATE int Size;

#define ORG_MODE 0
#define GET_MODE 1
#define POLL_MODE 2

PRIVATE BOOL TermMode = ORG_MODE;


/*******************************************************************
********************************************************************/

PUBLIC VOID HostEnd()
{
}


/*******************************************************************
********************************************************************/

PUBLIC VOID ResetTerminal()
{
   if ( TermMode != ORG_MODE )
   {
         HostEnd();
         TermMode = ORG_MODE;
   }
}


/*******************************************************************
********************************************************************/

PUBLIC VOID HostBegin()
{
}


/*******************************************************************
                           GetAKey()
********************************************************************/

PUBLIC BYTE GetAKey()
{
   BYTE c = 0;

   (void)read( 0, &c, 1);
   return(c);
}


/*******************************************************************
                             SpGetKey()
********************************************************************/

PUBLIC VOID SpGetkey()
{
   BUFFER_DECLARATIONS;
   BYTE c;

   DEBUG(( "SP.GETKEY" ));
   INIT_BUFFERS;

   c = GetAKey();

   DEBUG(("key was %c",c));
   PUT_BYTE( SP_SUCCESS );
   PUT_BYTE( c );  
   REPLY;
}



/*******************************************************************
                            SpPollkey()
********************************************************************/

PUBLIC VOID SpPollkey( void )
{
   BUFFER_DECLARATIONS;
   char c ;

   DEBUG(( "SP.POLLKEY" ));
   INIT_BUFFERS;

   if ( read( stdin, &c, 1) < 1 )
   {
         PUT_BYTE( SP_ERROR );
   }
   else
   {
         PUT_BYTE( SP_SUCCESS );
         PUT_BYTE( c );  
   }

   REPLY;
}



/*******************************************************************
                               SpGetenv()
********************************************************************/

PUBLIC VOID SpGetenv( void )
{
   BUFFER_DECLARATIONS;
   BYTE *Name;

   DEBUG(( "SP.GETENV" ));
   INIT_BUFFERS;

   Name = &DataBuffer[0];
   GET_SLICE( Size, Name );
   *(Name+Size)=0;
   DEBUG(( "\"%s\"", Name ));

   if ( *Name == 0 )
   {
         PUT_BYTE( SP_ERROR );
   }
   else
   {
        if ( ( Name=(BYTE *)getenv( Name ) ) == NULL )
		{
            PUT_BYTE( SP_ERROR );
        }
        else
        {
            DEBUG(( "\"%s\"", Name ));
            PUT_BYTE( SP_SUCCESS );
            Size = strlen( Name );
            PUT_SLICE( Size, Name );
        }
   }

   REPLY;
}


/*******************************************************************
                                 SpTime()
********************************************************************/

PUBLIC VOID SpTime( void )
{
   BUFFER_DECLARATIONS;
   long Time, UTCTime;

   DEBUG(( "SP.TIME" ));
   INIT_BUFFERS;

   time( &Time );
   UTCTime = 0L;
   PUT_BYTE( SP_SUCCESS );
   PUT_INT32( Time );
   PUT_INT32( UTCTime );
   REPLY;
}


/*******************************************************************
                              SpSystem()
********************************************************************/

PUBLIC VOID SpSystem( void )
{
   BUFFER_DECLARATIONS;

   BYTE *Command;
   INT32 Status;

   DEBUG(( "SP.SYSTEM" ));
   INIT_BUFFERS;

   Command = &DataBuffer[0];
   GET_SLICE(Size,Command );
   *(Command+Size)=0; DEBUG(( "\"%s\"", Command ));
   Status = system( Command );
   DEBUG(( "status %ld", Status ));
   PUT_BYTE( SP_SUCCESS );
   PUT_INT32( Status );
   REPLY;
}


/*******************************************************************
                                SpExit()
********************************************************************/

PUBLIC int SpExit( void )
{
   BUFFER_DECLARATIONS;
   long Status;
   
   DEBUG(( "SP.EXIT" ));
   INIT_BUFFERS;
   
   GET_INT32( Status );
   DEBUG(( "%ld", Status ));

   if( Status == 999999999 ) Status = HUNKY_DORY;
   if( Status == -999999999 ) Status = BAD_KARMA;

   DEBUG(( "exit with %d", (int)Status ));

   PUT_BYTE( SP_SUCCESS );
   REPLY ( (int)Status );
}

/********************************  EOF  *****************************/

