/*
 --  ---------------------------------------------------------------------------
 --
 --      ISERVER  -  INMOS standard file server
 --
 --      pack.h
 --
 --      Macros to cut and paste the Tbuf
 --
 --      Copyright (c) INMOS Ltd., 1988.
 --      All Rights Reserved.
 --
 --  ---------------------------------------------------------------------------
*/


#ifndef	PACK_INCLUDED
#define	PACK_INCLUDED

#   include "inmos.h"


/*
 *  some stuff used in processing all commands
 */

#define BUFFER_DECLARATIONS register BYTE *InBuf, *OutBuf; register int OutCount
#define INIT_BUFFERS InBuf = &Tbuf[3]; OutBuf = &Tbuf[2]; OutCount = 0
#define REPLY {Tbuf[0]= OutCount &0xFF; Tbuf[1]= (OutCount>>8) &0xFF; }; return


/*
 *   PACKING ROUTINES
 */


/*
 *   pack one byte with int x 
 */

#define PUT_BYTE(x) {*OutBuf++ = x; OutCount++;}


/*
 *  pack lower 16 bits of int x into 2 bytes
 */

#define PUT_INT16(x) {BYTE *c; c=(BYTE *)&x; *OutBuf++ = *c++; *OutBuf++ = *c; OutCount += 2;}


/*
 *  pack int32 into 4 bytes
 */

#define PUT_INT32(x) {BYTE *c; c=(BYTE *)&x; *OutBuf++ = *c++; *OutBuf++ = *c++;*OutBuf++ = *c++; *OutBuf++ = *c; OutCount += 4;}


/*
 *  pack FILE *x as 4 bytes
 */

#define PUT_FD(x) PUT_INT32(x)


/*
 *  pack lower 16 bits of int x as 2 bytes and then pack x bytes from y
 */

#define PUT_SLICE(x,y) {BYTE *c; c=(BYTE *)&x; *OutBuf++ = *c++; *OutBuf++ = *c; memcpy(OutBuf, (BYTE *)y, x); OutBuf += x; OutCount += x+2; }


/*
 *  pack lower 16 bits of int x into bytes 0 and 1 of Tbuf
 */

#define PUT_COUNT(x) {BYTE *c; c=(BYTE *)&(x); Tbuf[0]= *c++; Tbuf[1]= *c; }


/*
 *   UNPACKING ROUTINES
 */


/*
 *  unpack one byte into int x
 */

#define GET_BYTE(x) x = *InBuf++


/*
 *  unpack 2 bytes into int x
 */

#define GET_INT16(x) {BYTE *c; c=(BYTE *)&x; *c++ = *InBuf++; *c = *InBuf++; }


/*
 *  unpack 4 bytes into int32 x
 */

#define GET_INT32(x) {BYTE *c; c=(BYTE *)&x; *c++ = *InBuf++; *c++ = *InBuf++; *c++ = *InBuf++; *c = *InBuf++; }


/*
 *  unpack 4 bytes into FILE *x
 */

#define GET_FD(x) GET_INT32(x)


/*
 *  unpack 2 bytes into int x and then x bytes into array y
 */

#define GET_SLICE(x,y) {x= *InBuf++; x += (256*(*InBuf++)); memcpy(y, InBuf, x); InBuf += x;}

#endif

/*********************************  EOF  *********************************/

